CON _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000    

    SI      = 31                ' serial port in
    SO      = 30                ' serial port out
    doPin   = 12                ' SD card data out
    clkPin  = 13                ' SD card clock
    diPin   = 14                ' SD card data in
    csPin   = 15                ' SD card chips select
    audioR  = 24                ' right audio channel
    audioL  = 0                 ' left audio channel (0 if mono)

VAR
  byte  AYregs[16]
  byte  buffer[128]
  byte  dirent[fat#dent_bytes]
  byte  filename[13]
  long  nvbl
  long  attr
  long  ndrums
  long  clock
  long  rate
  long  loop
  long  extra
  long  bytes
  long  fileNumber
  long  nextsong
  long  wait

OBJ
  SID   : "AYcog_V0-21"
  FAT   : "fatfs"
  FDX   : "pcFullDuplexSerial2FC"

PUB Main | okay, dent, fp, ch

  okay := \fat.start(doPin, clkPin, diPin, csPin)
  okay := SID.start( audioR, audioL, @AYregs )
  fdx.Init
  fdx.AddPort(0,SI,SO,-1,-1,0,0,115200)
  okay := fdx.Start

  okay := \fat.mount
  if okay < 0
    fdx.str(0,string("Can't mount SD card",13,10))
    return

  dent := fat.find_file(string("YM"))
  if dent => 0
    fat.chdir(dent)
  fileNumber := 0
  nextsong := 1
  repeat
    if nextsong
      fileNumber++
      fat.fclose(fp)
      fp := openNextFile
      nextsong := 0
    waitcnt(CNT + clkFreq / rate)
    if fat.fread(fp, @buffer, 16) <> 16
      nextsong := 1
      next
    if fdx.rxavail(0)
      ch := fdx.rx(0)
      fdx.tx(0, ch)
      nextsong := 1
      next
    if buffer[0] == "E" and buffer[1] == "n" and buffer[2] == "d" and buffer[3] == "!"
      nextsong := 1
      next
    bytemove( @AYregs, @buffer, 13 )
    ifnot buffer[13] == 255
      byte[@AYregs + 13] := buffer[13]&15

PUB openNextFile : fp | rec, i, j, m, n
  rec := fat.opendir(@dirent)
  i := fileNumber
  repeat
    if dirent[8] == "Y" and dirent[9] == "M" and dirent[10] == " "
      if i-- == 0
        quit
    rec := fat.readdir(rec, @dirent)
    if rec < 0
      return -1

  fat.get_dent_name(@dirent, @filename)
  fdx.str(0,string("Now playing "))
  fdx.str(0,@filename)
  fdx.str(0,string(13,10))
  fp := \fat.fopen(@filename, "r")
  if fp < 0
    return -1
  fat.fread(fp, @buffer, 4)        ' 00 - read "YM6!" header
  fat.fread(fp, @buffer, 8)        ' 04 - read "LeOnArD!" identifier
  nvbl := get_dword(fp)            ' 0c - number of VBL frames
  fdx.str(0,string("VBL frames : "))
  fdx.dec(0,nvbl)
  crlf
  attr := get_dword(fp)            ' 10 - attributes
  fdx.str(0,string("Attributes : "))
  fdx.hex(0,attr,8)
  crlf
  ndrums := get_word(fp)           ' 14 - number of digi drum samples
  fdx.str(0,string("Digi-Drums : "))
  fdx.dec(0,ndrums)
  crlf
  clock := get_dword(fp)           ' 16 - YM2149 external frequency in Hz
  fdx.str(0,string("Clock      : "))
  fdx.dec(0,clock)
  crlf
  rate := get_word(fp)             ' 1a - player frequency in Hz
  fdx.str(0,string("Play rate  : "))
  fdx.dec(0,rate)
  crlf
  loop := get_dword(fp)            ' 1c - frame where looping starts
  fdx.str(0,string("Loop       : "))
  fdx.dec(0,loop)
  crlf
  extra := get_word(fp)            ' 20 - extra bytes following (should be 0)
  fdx.str(0,string("Extra      : "))
  fdx.dec(0,extra)
  crlf

  ' skip over extra data
  repeat while extra > 0
    m := extra <# 128
    fat.fread(fp, @buffer, m)
    extra -= m
  if ndrums > 0
    ' read digi drum samples
    repeat n from 0 to ndrums-1
      bytes := get_dword(fp)
      repeat while bytes > 0
        m := bytes <# 128
        fat.fread(fp, @buffer, m)
        bytes -= m

  get_string(fp)                  ' read song name
  fdx.str(0,string("Song name  : "))
  fdx.str(0,@buffer)
  crlf
  get_string(fp)                  ' read author name
  fdx.str(0,string("Author     : "))
  fdx.str(0,@buffer)
  crlf
  get_string(fp)                  ' read comments / software
  fdx.str(0,string("Comments   : "))
  fdx.str(0,@buffer)
  crlf

PRI get_word(fp) | buff
  if fat.fread(fp, @buff, 2) <> 2
    return -1
  return byte[@buff][0] << 8 + byte[@buff][1]

PRI get_dword(fp) | buff
  if fat.fread(fp, @buff, 4) <> 4
    return -1
  return byte[@buff][0] << 24 + byte[@buff][1] << 16 + byte[@buff][2] << 8 + byte[@buff][3]

PRI get_string(fp) | n
  repeat n from 0 to 127
    if (buffer[n] := fat.fgetc(fp)) =< 0
      buffer[n] := 0
      return

PRI crlf
  fdx.str(0,string(13,10))
